import sys, os
import json
import database_query as dq
import rsa_helper as rh
import locator
from configuration import Configuration


"""
Creates a link file from existing data (that was created with the insert_data.py)
"""

def print_help():
    print('Usage:')
    print('')
    print('python3 get_link_from_data.py dataname')
    print('python3 get_link_from_data.py --testrun')

    sys.exit()


# Takes 'Name' entry from database and generates a link named 'name'.10f which contains
# IP, PORT, NAME, ORIGFILENAME, AES&IV, RSA_PUB
# AES is stored in 64 bytes hex string
# the link is saved on disc.
def create_and_save_link(name):
    print('Creating link:', name)

    row = dq.get_link_data_by_name(name)
    keypair = dq.get_rsa_hex_keys()

    if row is not None and keypair is not None:
        prv, pub = keypair
        # Get our local ip:port address (for now it's manual text file)
        # Get config information
        config = Configuration()

        #with open(os.path.join(locator.module_path(), 'config.json'), 'r') as f:
        #    config = json.load(f)

        # todo: add the IP:PORT:RSA_PUB for all the fingers pointing to this data

        # Create the json link:
        address_list = []
        # Add 'my' address
        ip = config.get('ip', '127.0.0.1')
        port = config.get('port', 10000)
        address_list.append({'rsa_pub': pub, 'ip': ip, 'port': port})

        # todo: add fingers addresses:
        # Get data id:
        dataid = dq.get_link_dataid_by_name(name)

        print("FINGERS:", name, 'dataid:', dataid, flush=True)


        # Get a list of finger id
        fingers = dq.get_all_fingers(dataid)

        if fingers == 0:
            print('This link have not yet been uploaded elsewhere!')

        print("Fingers:", len(fingers), flush=True)

        fingers_len = len(fingers)
        print("FINGERS 0 len=", fingers_len, flush=True)
        for n in range(0, fingers_len):
            print("FINGERS 1", n, flush=True)
            fingerid = fingers[n]
            print("FINGERS 2", type(fingerid), flush=True)
            # add to the address list:
            print("FINGERS 3 fingerid=", fingerid, flush=True)
            nodeid, dataid_check = dq.get_finger_data(fingerid)
            print("FINGERS 4 dataid:", dataid, "dataid_check:", dataid_check, "nodeid:", nodeid, flush=True)

            if dataid is not dataid_check:
                print("#Error in SQL, dataid is not dataid_check")
            if nodeid is not 0:
                # get rsa_pub, ip, port from node and add it to the address list
                node_data = dq.get_node_data(nodeid)
                address_list.append({'rsa_pub': node_data[2], 'ip': node_data[0], 'port': node_data[1]})

        # Get the new, encrypted, address:
        encrypted_address = dq.get_encrypted_addresses(dataid)


        print("FINGERS X", flush=True)
        # generate link data
        # data_rsa is, together with the name, the unique identifier for this data (e.g the rsa_pub key from the node owning the data)
        _, data_rsa_pub = rh.get_rsa_keypair(True)
        data_rsa_pub = data_rsa_pub.exportKey('DER').hex()

        d = {'name': name, 'originalfilename': row[1], 'key': row[2], 'iv': row[3], 'data_rsa_pub': data_rsa_pub,
             'address_list': address_list, 'encrypted_address_list': encrypted_address}

        # todo: sign the link

        s = json.dumps(d)
        beautify = True  # Insert linefeeds in json
        if beautify:
            s = s.replace("}", "}\n")
        with open(name+'.10f', 'w') as f:
            f.write(s)

        print('Link created.')
    else:
        if row is None:
            print('#Error: Link not found in datbase.')
        if keypair is None:
            print('#Error: RSA kaypair not found in datbase.')


# Test case:
def test():
    print('[get_link_from_data.py]')
    create_and_save_link('test')
    create_and_save_link('image')



# Call the main function with the command line arguments
if __name__ == '__main__':
    if len(sys.argv) == 1:
        print_help()

    if sys.argv[1] == '--testrun':
        test()
    else:
        create_and_save_link(sys.argv[1])
